#!/bin/dash
# fatdog-battery-applet.sh, replacement for powerapplet
# display an icon on icon tray showing how much battery charge is left
# (C) James Budiono 2014
# License: GPL Version 3 or later
# Plastic surgery by dr. Dan 2020
#
# uses 'sit' (simple-icon-tray) from technosaurus
#
# Note: doesn't deal with multiple power supplies and multiple batteries,
# only the first one is shown, but filling and draining should reflect sum of all batteries.

### configuration
CHECK_EVERY=60 #seconds
POWER_PATH=/sys/class/power_supply
IMG_FILE=/tmp/batt.$$.svg
TOOLTIP=/tmp/batt.$$.tip
ACTION=/tmp/batt.$$.action
LOWBATT=15 CRITICALBATT=5 SHUTDOWN=2 
LOWBATT_ACTION='batt-low.sh low'
CRITICALBATT_ACTION='batt-low.sh critical'
CLICK_ACTION="Xdialog --left --title \"Battery Info\" --msgbox \"\$(cat $POWER_PATH/*/uevent | sed 's/=/:    /')\" 0 0"
RIGHT_CLICK_ACTION="fatdog-power-settings.sh" # configuration
[ -e /etc/battery.conf ] && . /etc/battery.conf # get preset configuration
[ -e $FATDOG_STATE_DIR/battery.conf ] && . $FATDOG_STATE_DIR/battery.conf #user overrides

BATTERY_SVG='<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<svg version="1.0" width="60" height="60" x="0" y="0">
  <rect id="body"   x="14" y="15" height="40" width="32" rx="3" ry="3" style="fill:white;stroke:black;stroke-width:3;"/>
  <rect id="charge" x="15" y="16" height="38" width="30" rx="3" ry="3" style="fill:red;stroke:none;"/>
  <rect id="cap"    x="22" y="9"  height="6"  width="16" rx="3" ry="3" style="fill:white;stroke:black;stroke-width:3;"/>
  <path id="online" d="M26,20 H38 L32,30 L36,38 L22,50 L26,36 L20,32 z" style="fill:yellow;stroke:black;stroke-width:2" />
  <path id="absent" d="M20,25 L40,45 M20,45 L40,25" style="stroke:red;stroke-width:4;"/>
</svg>'
CHARGE_MAX=38
CHARGE_OFF=16

# $1-online $2-capacity $3-timestatus $4-percent of original capacity $5-current capacity $6-original capacity
update_image() {
	# charges
	# tooltip
	if [ $1 ] && [ $1 = "0" ]; then
		if [ -z "$2" ]; then
			if [ -z "$4" ]; then
				echo -n "Unknown portion of battery left. \nLow @ $LOWBATT%  Critical @ $CRITICALBATT%  Shutdown @ $SHUTDOWN%" > $TOOLTIP
			else
				echo -n "Unknown portion of battery left. \nLow @ $LOWBATT%  Critical @ $CRITICALBATT%  Shutdown @ $SHUTDOWN% \nBattery health @ ${4}% ${numfmt --to=si ${5}/M/m}Ah/${numfmt --to=si ${6}/M/m}Ah" > $TOOLTIP
			fi
		else			
			if [ $3 ] && [ "$3" != "-1" ]; then
				if [ -z "$4" ]; then
					echo -n "${2}% battery left.  About ${3} minutes left. \nLow @ $LOWBATT%  Critical @ $CRITICALBATT%  Shutdown @ $SHUTDOWN%" > $TOOLTIP
				else
					echo -n "${2}% battery left.  About ${3} minutes left. \nLow @ $LOWBATT%  Critical @ $CRITICALBATT%  Shutdown @ $SHUTDOWN% \nBattery health @ ${4}%:  ${numfmt --to=si ${5}/M/m}Ah/${numfmt --to=si ${6}/M/m}Ah" > $TOOLTIP
				fi
			else
				if [ -z "$4" ]; then
					echo -n "${2}% battery left. \nLow @ $LOWBATT%  Critical @ $CRITICALBATT%  Shutdown @ $SHUTDOWN%" > $TOOLTIP
				else
					echo -n "${2}% battery left. \nLow @ $LOWBATT%  Critical @ $CRITICALBATT%  Shutdown @ $SHUTDOWN% \nBattery health @ ${4}%:  ${(numfmt --to=si ${5})/M/m}Ah/$({numfmt --to=si ${6})/M/m}Ah" > $TOOLTIP
				fi
			fi
		fi
	else
		if [ $1 ] && [ $1 = "1" ]; then
			if [ "$3" ] && [ "$3" = "-1" ]; then
				echo -n "Charging, ${2}% battery level. \nLow @ $LOWBATT%  Critical @ $CRITICALBATT%  Shutdown @ $SHUTDOWN% \nBattery health @ ${4}%:  ${numfmt --to=si ${5}/M/m}Ah/${numfmt --to=si ${6}/M/m}Ah" > $TOOLTIP
			else
				if [ "$3" ] && [ $3 = "full" ]; then
					echo -n "Fully charged. \nLow @ $LOWBATT%  Critical @ $CRITICALBATT%  Shutdown @ $SHUTDOWN% \nBattery health @ ${4}% ${numfmt --to=si ${5}/M/m}Ah/${numfmt --to=si ${6}/M/m}Ah" > $TOOLTIP
				else
					echo -n "Charging, ${2}% battery level.  About ${3} minutes to full. \nLow @ $LOWBATT%  Critical @ $CRITICALBATT%  Shutdown @ $SHUTDOWN% \nBattery health @ ${4}%:  ${numfmt --to=si ${5}/M/m}Ah/${numfmt --to=si ${6}/M/m}Ah" > $TOOLTIP 
				fi
			fi
		fi
	fi
	# icon
	charge_sed="/charge/d" colour_sed="" absent_sed="/absent/d"
	if [ $2 ]; then
		charge=$(($2*CHARGE_MAX/100))
		offset=$((CHARGE_OFF+CHARGE_MAX-charge))
		charge_sed="/charge/ {s|y=\"$CHARGE_OFF\"|y=\"$offset\"|; s|height=\"$CHARGE_MAX\"|height=\"$charge\"|}"
		[ $2 -ge 30 ] && colour_sed="/charge/ s|:red|:cyan|"		
		[ $2 -ge 70 ] && colour_sed="/charge/ s|:red|:green|"
	fi
	
	# ac power
	power_sed="/online/d"
	[ "$1" ] && [ $1 -eq 1 ] && power_sed=""
	
	# output image
	echo "$BATTERY_SVG" | sed -e "$charge_sed" -e "$colour_sed" -e "$power_sed" -e "$absent_sed" > $IMG_FILE
	
	# check battery low
	if [ "$power_sed" ] && [ "$2" ] && [ $2 -le $LOWBATT ]; then
		if [ $2 -le $CRITICALBATT ]; then 
			$CRITICALBATT_ACTION & # don't block
		else 
		$LOWBATT_ACTION & # don't block
		fi
	fi
}

charging_status() {
	previous=$1 # retain level from a minute ago current level
	if [ -s $POWER_PATH/*/charge_now ] ; then
			read chargenow < $POWER_PATH/*/charge_now # current level
	else [ -s $POWER_PATH/*/energy_now ] && read chargenow < $POWER_PATH/*/energy_now # current level
	fi
	unfilled=$((full-chargenow))
	if [ "$unfilled" -ne 0 ]; then
		change=$((chargenow-previous)) # change over last minute (sometimes results in zero change)
		if [ $change -ne 0 ]; then
			timestatus=$((unfilled/change))
		else timestatus=-1
		fi
	else
		timestatus="full"
	fi
}

draining_status() {
	previous=$1 # retain level from a minute ago
	# chargenow=$(cat $POWER_PATH/*/charge_now 2>/dev/null | head -n 1) || chargenow=$(cat $POWER_PATH/*/energy_now 2>/dev/null | head -n 1) # current level
	if [ -s $POWER_PATH/*/charge_now ] ; then
			read chargenow < $POWER_PATH/*/charge_now # current level
	else [ -s $POWER_PATH/*/energy_now ] && read chargenow < $POWER_PATH/*/energy_now # current level
	fi
	change=$((previous-chargenow)) #  dr. Dan - change over last minute (sometimes results in zero change)
	if [ $change -gt 0 ]; then
		timestatus=$((chargenow/change))
	else timestatus=-1
	fi
}

### pause until the battery is re-installed
battery_removed() {
	power_sed="/online/d" colour_sed="/charge/ s|:red|:white|"
	charge_sed="/charge/ {s|y=\"$CHARGE_OFF\"|y=\"15\"|; s|height=\"$CHARGE_MAX\"|height=\"38\"|}"
	absent_sed=""
	kill -STOP $XPID # SFR - continue sit
	echo "$BATTERY_SVG" | sed -e "$charge_sed" -e "$colour_sed" -e "$power_sed" -e "$absent_sed" > $IMG_FILE
	echo "Battery removed" > $TOOLTIP
	kill -CONT $XPID # SFR - continue sit
	while [ "$capacity" = "" ]; do
		# capacity=$(cat $POWER_PATH/*/capacity 2>/dev/null | head -n 1)
		[ -s $POWER_PATH/*/capacity ] && read capacity < $POWER_PATH/*/capacity
		sleep $CHECK_EVERY
	done
	health_check
}

health_check() {
	# full=$(cat $POWER_PATH/*/charge_full 2>/dev/null | head -n 1) || full=$(cat $POWER_PATH/*/energy_full 2>/dev/null | head -n 1)
	# design=$(cat $POWER_PATH/*/charge_full_design 2>/dev/null | head -n 1) || design=$(cat $POWER_PATH/*/energy_full_design 2>/dev/null | head -n 1)
	if [ -s $POWER_PATH/*/charge_full ] ; then
			read full < $POWER_PATH/*/charge_full # current level
	else [ -s $POWER_PATH/*/energy_full ] && read full < $POWER_PATH/*/energy_full # current level
	fi
	if [ -s $POWER_PATH/*/charge_full_design ] ; then
			read design < $POWER_PATH/*/charge_full_design # current level
	else [ -s $POWER_PATH/*/energy_full_design ] && read design < $POWER_PATH/*/energy_full_design # current level
	fi
	if [ $full ] && [ $design ]; then
		: $((health= full *100 / design)) # current capacity vs. original
	fi
}	

### main ###
update_image 0
touch $TOOLTIP
printf "%s\n%s &" "#!/bin/dash" "$CLICK_ACTION" > $ACTION; chmod +x $ACTION
sit $IMG_FILE $TOOLTIP $ACTION $RIGHT_CLICK_ACTION &
XPID=$!
trap '{ kill $XPID; rm $IMG_FILE $TOOLTIP $ACTION; } 2>/dev/null; exit' INT TERM HUP 0
chargenow=0
if [ -s $POWER_PATH/*/charge_now ] ; then
	read chargenow < $POWER_PATH/*/charge_now # current level
else [ -s $POWER_PATH/*/energy_now ] && read chargenow < $POWER_PATH/*/energy_now # current level
fi
online=0 capacity=100 full=1 design=1 health="unknown"
health_check
while :; do
	formerstate=$online
	# online=$(cat $POWER_PATH/*/online 2>/dev/null | head -n 1) # evaluate power state
	[ -s $POWER_PATH/*/online ] && read online < $POWER_PATH/*/online
	# capacity=$(cat $POWER_PATH/*/capacity 2>/dev/null | head -n 1)
	[ -s $POWER_PATH/*/capacity ] && read capacity < $POWER_PATH/*/capacity
	if [ -z "$capacity" ]; then # can't find capacity measurement, pause and trigger a battery state change.
		formerstate=2
		battery_removed
	fi
	if [ $formerstate -ne $online ]; then # avoids odd numbers when connecting or disconnecting power
		# chargenow=$(cat $POWER_PATH/*/charge_now 2>/dev/null | head -n 1) || chargenow=$(cat $POWER_PATH/*/energy_now 2>/dev/null | head -n 1)
		if [ -s $POWER_PATH/*/charge_now ] ; then
			read chargenow < $POWER_PATH/*/charge_now # current level
		else [ -s $POWER_PATH/*/energy_now ] && read chargenow < $POWER_PATH/*/energy_now # current level
		fi
	fi
	if [ $online -eq 1 ]; then
		charging_status "$chargenow"
		else
		draining_status	"$chargenow"
	fi
	kill -0 $XPID || exit # if sit dies, we leave too
	kill -STOP $XPID # SFR - pause sit
	update_image "$online" "$capacity" "$timestatus" "$health" "$full" "$design" # for time remaining and health of battery
	kill -CONT $XPID # SFR - continue sit
	sleep $CHECK_EVERY
done
